@@ -42,7 +42,7 @@ class CourseraDownloader(object):
     # how long to try to open a URL before timing out
     TIMEOUT=60.0
 
-    def __init__(self,username,password,proxy=None,parser=DEFAULT_PARSER,ignorefiles=None):
+    def __init__(self,username,password,proxy=None,parser=DEFAULT_PARSER,ignorefiles=None, max_path_len=None):
         self.username = username
         self.password = password
         self.parser = parser
@@ -54,6 +54,7 @@ class CourseraDownloader(object):
 
         self.browser = None
         self.proxy = proxy
+        self.max_path_len = max_path_len
 
     def login(self,className):
         """
@@ -191,7 +192,8 @@ class CourseraDownloader(object):
 
                 for a in hrefs:
                     # get the hyperlink itself
-                    h = a['href']
+                    h = a.get('href')
+                    if not h: continue
 
                     # Sometimes the raw, uncompresed source videos are available as
                     # well. Don't download them as they are huge and available in
@@ -209,17 +211,25 @@ class CourseraDownloader(object):
                 if not hasvid:
                     ll = li.find('a',{'class':'lecture-link'})
                     lurl = ll['data-modal-iframe']
-                    pg = self.browser.open(lurl,timeout=self.TIMEOUT)
-                    bb = BeautifulSoup(pg,self.parser)
-                    vobj = bb.find('source',type="video/mp4")
 
-                    if not vobj:
-                        print " Warning: Failed to find video for %s" %  className
-                    else:
-                        vurl = vobj['src']
-                        # build the matching filename
-                        fn = className + ".mp4"
-                        resourceLinks.append( (vurl,fn) )
+                    try:
+                        pg = self.browser.open(lurl,timeout=self.TIMEOUT)
+
+                        bb = BeautifulSoup(pg,self.parser)
+                        vobj = bb.find('source',type="video/mp4")
+
+                        if not vobj:
+                            print " Warning: Failed to find video for %s" %  className
+                        else:
+                            vurl = vobj['src']
+                            # build the matching filename
+                            fn = className + ".mp4"
+                            resourceLinks.append( (vurl,fn) )
+
+                    except urllib2.HTTPError as e:
+                        # sometimes there is a lecture without a vidio (e.g.,
+                        # genes-001) so this can happen.
+                        print " Warning: failed to open the direct video link %s: %s" % (lurl,e)
 
                 weekClasses[className] = resourceLinks
 
@@ -261,6 +271,9 @@ class CourseraDownloader(object):
 
         filepath = path.join(target_dir,fname)
 
+        if self.max_path_len:
+            filepath = trim_path(filepath,max_path_len=self.max_path_len)
+
         dl = True
         if path.exists(filepath):
             if clen > 0: 
@@ -442,6 +455,31 @@ def sanitise_filename(fileName):
     s = fn[:max-len(ext)] + ext
 
     return s
+    
+def trim_path(pathname, max_path_len=255, min_len=5):
+    """
+    Trim file name in given path name to fit max_path_len characters. Only file name is trimmed,
+    path names are not affected to avoid creating multiple folders for the same lecture.
+    """
+    if len(pathname) <= max_path_len:
+        return pathname
+
+    fpath, name = path.split(pathname)
+    name, ext = path.splitext(name)
+
+    to_cut = len(pathname) - max_path_len
+    to_keep = len(name) - to_cut
+
+    if to_keep < min_len:
+        print ' Warning: Cannot trim filename "%s" to fit required path length (%d)' % (pathname, max_path_len)
+        return pathname
+
+    name = name[:to_keep]
+    new_pathname = path.join(fpath, name + ext)
+    print ' Trimmed path name "%s" to "%s" to fit required length (%d)' % (pathname, new_pathname, max_path_len)
+
+    return new_pathname
+
 
 # TODO: simplistic
 def isValidURL(url):
@@ -558,17 +596,18 @@ def main():
                         default=False, help="download and save the sections in reverse order")
     parser.add_argument('course_names', nargs="+", metavar='<course name>',
                         type=str, help='one or more course names from the url (e.g., comnets-2012-001)')
+    parser.add_argument("--trim-path", dest='trim_path', action='store_true', default=True, help='Trim path names to fit OS constraints (windows only)')
     args = parser.parse_args()
 
     # check the parser
-    parser = args.parser
-    if parser == 'lxml' and not haslxml():
+    html_parser = args.parser
+    if html_parser == 'lxml' and not haslxml():
         print " Warning: lxml not available, falling back to built-in 'html.parser' (see -q option), this may cause problems on Python < 2.7.3"
-        parser = 'html.parser'
+        html_parser = 'html.parser'
     else:
         pass
 
-    print "Coursera-dl v%s (%s)" % (_version.__version__,parser)
+    print "Coursera-dl v%s (%s)" % (_version.__version__,html_parser)
 
     # search for login credentials in .netrc file if username hasn't been provided in command-line args
     username, password = args.username, args.password
@@ -583,9 +622,19 @@ def main():
         if not password:
             password = getpass.getpass()
 
+    # should we be trimming paths?
+    max_path_len = None
+    if args.trim_path:
+        if platform.system() == "Windows":
+            max_path_len = 257
+            print "Maximum path length set to %s" % max_path_len
+        else:
+            # linux max path length is typically around 4060 so assume thats ok
+            pass   
+ 
     # instantiate the downloader class
-    d = CourseraDownloader(username,password,proxy=args.proxy,parser=parser,ignorefiles=args.ignorefiles)
-    
+    d = CourseraDownloader(username,password,proxy=args.proxy,parser=html_parser,ignorefiles=args.ignorefiles,max_path_len=max_path_len)
+
     # authenticate, only need to do this once but need a classaname to get hold
     # of the csrf token, so simply pass the first one
     print "Logging in as '%s'..." % username
